@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003      Imendio HB
+ * Copyright (C) 2003-2004 Imendio HB
  * Copyright (C) 2002-2003 Richard Hult <richard@imendio.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -90,6 +90,71 @@ gossip_option_menu_setup (GtkWidget     *option_menu,
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
 }
 
+void
+gossip_option_image_menu_setup (GtkWidget     *option_menu,
+				GCallback      func,
+				gpointer       user_data,
+				gconstpointer  str1, ...)
+{
+	GtkWidget     *menu;
+	GtkWidget     *item;
+	GtkWidget     *hbox;
+	GtkWidget     *label;
+	GtkWidget     *image;
+	gint           i;
+	va_list        args;
+	gconstpointer  str;
+	gint           type;
+	
+       	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (option_menu));
+	if (menu) {
+		gtk_widget_destroy (menu);
+	}
+	
+	menu = gtk_menu_new ();
+
+	va_start (args, str1);
+	for (str = str1, i = 0; str != NULL; str = va_arg (args, gpointer), i++) {
+		if (*(gchar *)str == '\0') {
+			item = gtk_separator_menu_item_new ();
+			str = va_arg (args, gpointer);
+		} else {
+			item = gtk_menu_item_new ();
+
+			hbox = gtk_hbox_new (FALSE, 4);
+
+			label = gtk_label_new (str);
+
+			str = va_arg (args, gpointer);
+			image = gtk_image_new_from_stock (str, GTK_ICON_SIZE_MENU);
+
+			gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+			gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
+			
+			gtk_container_add (GTK_CONTAINER (item), hbox);
+		}
+
+		gtk_widget_show_all (item);
+		gtk_menu_append (GTK_MENU (menu), item);
+
+		type = va_arg (args, gint);
+		
+		g_object_set_data (G_OBJECT (item),
+				   "data",
+				   GINT_TO_POINTER (type));
+		if (func) {
+			g_signal_connect (item,
+					  "activate",
+					  func,
+					  user_data);
+		}
+	}
+	va_end (args);
+
+	gtk_widget_show (menu);
+	gtk_option_menu_set_menu (GTK_OPTION_MENU (option_menu), menu);
+}
+
 void
 gossip_option_menu_set_history (GtkOptionMenu *option_menu,
 				gpointer       user_data)
@@ -665,8 +730,8 @@ gossip_utils_get_pixbuf_offline (void)
 	return gossip_utils_get_pixbuf_from_stock (GOSSIP_STOCK_OFFLINE);
 }
 
-GdkPixbuf *
-gossip_utils_get_pixbuf_from_show (GossipShow show)
+const gchar *
+gossip_utils_get_stock_from_show (GossipShow show)
 {
 	const gchar *stock = NULL;
 	
@@ -685,6 +750,15 @@ gossip_utils_get_pixbuf_from_show (GossipShow show)
 		break;
 	}
 
+	return stock;
+}
+
+GdkPixbuf *
+gossip_utils_get_pixbuf_from_show (GossipShow show)
+{
+	const gchar *stock;
+
+	stock = gossip_utils_get_stock_from_show (show);
 	return gossip_utils_get_pixbuf_from_stock (stock);
 }
 
@@ -704,57 +778,104 @@ gossip_utils_get_default_status (GossipShow show)
 	return _(AVAILABLE_MESSAGE);
 }
 
-GSList *
-gossip_utils_get_busy_messages (void)
+GList *
+gossip_utils_get_status_messages (void)
 {
-	GConfValue *value;
-	GSList     *list;
+	GSList            *list, *l;
+	GList             *ret = NULL;
+	GossipStatusEntry *entry;
+	const gchar       *status;
 	
-	/* FIXME: Workaround for broken intltool for now... It clears the
-	 * default values for lists. See #121330.
-	 */
-	value = gconf_client_get (gconf_client,
-				  "/apps/gossip/status/custom_busy_messages",
-				  NULL);
-	
-	if (!value) {
-		list = g_slist_append (NULL, g_strdup (_("Working")));
-	} else {
-		gconf_value_free (value);
-		
-		list = gconf_client_get_list (gconf_client,
-					      "/apps/gossip/status/custom_busy_messages",
-					      GCONF_VALUE_STRING,
-					      NULL);
-	}
+	list = gconf_client_get_list (gconf_client,
+				      "/apps/gossip/status/preset_messages",
+				      GCONF_VALUE_STRING,
+				      NULL);
 
-	return list;
+	/* This is really ugly, but we can't store a list of pairs and a dir
+	 * with entries wouldn't work since we have no guarantee on the order of
+	 * entries.
+	 */
+	
+	for (l = list; l; l = l->next) {
+		entry = g_new (GossipStatusEntry, 1);
+
+		status = l->data;
+
+		if (strncmp (status, "available/", 10) == 0) {
+			entry->string = g_strdup (&status[10]);
+			entry->show = GOSSIP_SHOW_AVAILABLE;
+		}
+		else if (strncmp (status, "busy/", 5) == 0) {
+			entry->string = g_strdup (&status[5]);
+			entry->show = GOSSIP_SHOW_BUSY;
+		}
+		else if (strncmp (status, "away/", 5) == 0) {
+			entry->string = g_strdup (&status[5]);
+			entry->show = GOSSIP_SHOW_AWAY;
+		} else {
+			continue;
+		}
+		
+		ret = g_list_append (ret, entry);
+	}
+	
+	g_slist_foreach (list, (GFunc) g_free, NULL);
+	g_slist_free (list);
+	
+	return ret;
 }
 
-GSList *
-gossip_utils_get_away_messages (void)
+void
+gossip_utils_set_status_messages (GList *list)
 {
-	GConfValue *value;
-	GSList     *list;
+	GList             *l;
+	GossipStatusEntry *entry;
+	GSList            *slist = NULL;
+	const gchar       *show;
+	gchar             *str;
 	
-	/* FIXME: Workaround for broken intltool for now... It clears the
-	 * default values for lists. See #121330.
-	 */
-	value = gconf_client_get (gconf_client,
-				  "/apps/gossip/status/custom_away_messages",
-				  NULL);
-	
-	if (!value) {
-		list = g_slist_append (NULL, g_strdup (_("Eating")));
-		list = g_slist_append (list, g_strdup (_("Sleeping")));
-	} else {
-		gconf_value_free (value);
+	for (l = list; l; l = l->next) {
+		entry = l->data;
+
+		switch (entry->show) {
+		case GOSSIP_SHOW_AVAILABLE:
+			show = "available";
+			break;
+		case GOSSIP_SHOW_BUSY:
+			show = "busy";
+			break;
+		case GOSSIP_SHOW_AWAY:
+			show = "away";
+			break;
+		default:
+			show = NULL;
+			g_assert_not_reached ();
+		}
 		
-		list = gconf_client_get_list (gconf_client,
-					      "/apps/gossip/status/custom_away_messages",
-					      GCONF_VALUE_STRING,
-					      NULL);
+		str = g_strdup_printf ("%s/%s", show, entry->string);
+		slist = g_slist_append (slist, str);
 	}
 
-	return list;
+	gconf_client_set_list (gconf_client,
+			       "/apps/gossip/status/preset_messages",
+			       GCONF_VALUE_STRING,
+			       slist,
+			       NULL);
+
+	g_slist_foreach (slist, (GFunc) g_free, NULL);
+	g_slist_free (slist);
+}
+
+static void
+free_entry (GossipStatusEntry *entry)
+{
+	g_free (entry->string);
+	g_free (entry);
+}
+
+void
+gossip_utils_free_status_messages (GList *list)
+{
+	g_list_foreach (list, (GFunc) free_entry, NULL);
+	g_list_free (list);
 }
